@@ -38,13 +38,38 @@
 #include "sqliteInt.h"
 #include "btreeInt.h"
 #include "pager.h"
+#include "vdbeInt.h"
+
+#ifdef __ANDROID__
+#include <android/log.h>
+#endif
+
+#include <time.h>
+
+#if defined(_WIN32) || defined(SQLITE_OS_WINRT)
+#include <windows.h> /*  amalgamator: dontcache */
+#else
+#include <sys/time.h> /* amalgamator: dontcache */
+#endif
+
+#ifndef OMIT_MEMLOCK
+#if defined(__unix__) || defined(__APPLE__) || defined(_AIX)
+#include <errno.h> /* amalgamator: dontcache */
+#include <unistd.h> /* amalgamator: dontcache */
+#include <sys/resource.h> /* amalgamator: dontcache */
+#include <sys/mman.h> /* amalgamator: dontcache */
+#endif
+#endif
+
+#include "sqlcipher.h"
 
 /* extensions defined in pager.c */ 
-void *sqlite3PagerGetCodec(Pager*);
-void sqlite3PagerSetCodec(Pager*, void *(*)(void*,void*,Pgno,int),  void (*)(void*,int,int),  void (*)(void*), void *);
-int sqlite3pager_is_mj_pgno(Pager*, Pgno);
+void *sqlcipherPagerGetCodec(Pager*);
+void sqlcipherPagerSetCodec(Pager*, void *(*)(void*,void*,Pgno,int),  void (*)(void*,int,int),  void (*)(void*), void *);
+int sqlite3pager_is_sj_pgno(Pager*, Pgno);
 void sqlite3pager_error(Pager*, int);
 void sqlite3pager_reset(Pager *pPager);
+/* end extensions defined in pager.c */
 
 #if !defined (SQLCIPHER_CRYPTO_CC) \
    && !defined (SQLCIPHER_CRYPTO_LIBTOMCRYPT) \
@@ -59,7 +84,7 @@ void sqlite3pager_reset(Pager *pPager);
 #define CIPHER_STR(s) #s
 
 #ifndef CIPHER_VERSION_NUMBER
-#define CIPHER_VERSION_NUMBER 4.4.2
+#define CIPHER_VERSION_NUMBER 4.5.4
 #endif
 
 #ifndef CIPHER_VERSION_BUILD
@@ -111,49 +136,6 @@ void sqlite3pager_reset(Pager *pPager);
 #define CIPHER_MAX_KEY_SZ 64
 #endif
 
-#ifdef __ANDROID__
-#include <android/log.h>
-#endif
-
-#ifdef CODEC_DEBUG
-#ifdef __ANDROID__
-#define CODEC_TRACE(...) {__android_log_print(ANDROID_LOG_DEBUG, "sqlcipher", __VA_ARGS__);}
-#else
-#define CODEC_TRACE(...)  {fprintf(stderr, __VA_ARGS__);fflush(stderr);}
-#endif
-#else
-#define CODEC_TRACE(...)
-#endif
-
-#ifdef CODEC_DEBUG_MUTEX
-#define CODEC_TRACE_MUTEX(...)  CODEC_TRACE(__VA_ARGS__)
-#else
-#define CODEC_TRACE_MUTEX(...)
-#endif
-
-#ifdef CODEC_DEBUG_MEMORY
-#define CODEC_TRACE_MEMORY(...)  CODEC_TRACE(__VA_ARGS__)
-#else
-#define CODEC_TRACE_MEMORY(...)
-#endif
-
-#ifdef CODEC_DEBUG_PAGEDATA
-#define CODEC_HEXDUMP(DESC,BUFFER,LEN)  \
-  { \
-    int __pctr; \
-    printf(DESC); \
-    for(__pctr=0; __pctr < LEN; __pctr++) { \
-      if(__pctr % 16 == 0) printf("\n%05x: ",__pctr); \
-      printf("%02x ",((unsigned char*) BUFFER)[__pctr]); \
-    } \
-    printf("\n"); \
-    fflush(stdout); \
-  }
-#else
-#define CODEC_HEXDUMP(DESC,BUFFER,LEN)
-#endif
-
-/* end extensions defined in pager.c */
  
 /*
 **  Simple shared routines for converting hex char strings to binary data
@@ -191,6 +173,18 @@ static int cipher_isHex(const unsigned char *hex, int sz){
   return 1;
 }
 
+/* possible flags for simulating specific test conditions */
+#ifdef SQLCIPHER_TEST
+#define TEST_FAIL_ENCRYPT 0x01
+#define TEST_FAIL_DECRYPT 0x02
+#define TEST_FAIL_MIGRATE 0x04
+unsigned int sqlcipher_get_test_flags(void);
+void sqlcipher_set_test_flags(unsigned int);
+int sqlcipher_get_test_rand(void);
+void sqlcipher_set_test_rand(int);
+int sqlcipher_get_test_fail(void);
+#endif
+
 /* extensions defined in crypto_impl.c */
 /* the default implementation of SQLCipher uses a cipher_ctx
    to keep track of read / write state separately. The following
@@ -235,8 +229,8 @@ typedef struct {
 
 /* crypto.c functions */
 int sqlcipher_codec_pragma(sqlite3*, int, Parse*, const char *, const char*);
-int sqlite3CodecAttach(sqlite3*, int, const void *, int);
-void sqlite3CodecGetKey(sqlite3*, int, void**, int*);
+int sqlcipherCodecAttach(sqlite3*, int, const void *, int);
+void sqlcipherCodecGetKey(sqlite3*, int, void**, int*);
 void sqlcipher_exportFunc(sqlite3_context *, int, sqlite3_value **);
 
 /* crypto_impl.c functions */
@@ -329,6 +323,36 @@ int sqlcipher_find_db_index(sqlite3 *db, const char *zDb);
 
 int sqlcipher_codec_ctx_integrity_check(codec_ctx *, Parse *, char *);
 
+int sqlcipher_set_log(const char *destination);
+void sqlcipher_set_log_level(unsigned int level);
+void sqlcipher_log(unsigned int tag, const char *message, ...);
+
+#define SQLCIPHER_LOG_NONE          0x00
+#define SQLCIPHER_LOG_ERROR         0x01
+#define SQLCIPHER_LOG_WARN          0x02
+#define SQLCIPHER_LOG_INFO          0x04
+#define SQLCIPHER_LOG_DEBUG         0x08
+#define SQLCIPHER_LOG_TRACE         0x10
+#define SQLCIPHER_LOG_ALL           0xffffffff
+
+void sqlcipher_vdbe_return_string(Parse*, const char*, const char*, int);
+
+#ifdef CODEC_DEBUG_PAGEDATA
+#define CODEC_HEXDUMP(DESC,BUFFER,LEN)  \
+  { \
+    int __pctr; \
+    printf(DESC); \
+    for(__pctr=0; __pctr < LEN; __pctr++) { \
+      if(__pctr % 16 == 0) printf("\n%05x: ",__pctr); \
+      printf("%02x ",((unsigned char*) BUFFER)[__pctr]); \
+    } \
+    printf("\n"); \
+    fflush(stdout); \
+  }
+#else
+#define CODEC_HEXDUMP(DESC,BUFFER,LEN)
+#endif
+
 #endif
 #endif
 /* END SQLCIPHER */
